@@ -36,8 +36,10 @@ int cleanup_dbus(void)
 	if (dbus_error_is_set(&error)) {
 		dbus_error_free(&error);
 	}
-	dbus_connection_unref(connection);
-	connection=NULL;
+	if (connection) {
+		dbus_connection_unref(connection);
+		connection=NULL;
+	}
 	return 0;
 }
 
@@ -390,6 +392,11 @@ int check_dbus_messages(struct gps_device_t *devices, int max_devices)
 				set |= STATUS_SET;
 				continue;
 			}
+
+			if (!strcmp(status_str, "MODE_SET")) {
+				set |= MODE_SET;
+				continue;
+			}
 		}
 
 
@@ -427,6 +434,7 @@ int check_dbus_messages(struct gps_device_t *devices, int max_devices)
 			gps_mask_t changed = 0;
 			double online = 0;
 			int status = 0;
+			int mode = 0;
 
 			for (i=0; i<max_devices; i++) {
 				if (!(allocated_device(devices[i]))) {
@@ -448,6 +456,13 @@ int check_dbus_messages(struct gps_device_t *devices, int max_devices)
 					changed |= STATUS_SET;
 					gpsd_report(3, "Returning fix status\n");
 				}
+
+				if (set & MODE_SET) {
+					if (gpsdata->status)
+						mode = gpsdata->fix.mode;
+					changed |= MODE_SET;
+					gpsd_report(3, "Returning mode status\n");
+				}
 			}
 
 			if (changed) {
@@ -465,6 +480,12 @@ int check_dbus_messages(struct gps_device_t *devices, int max_devices)
 					else
 						device.gpsdata.status = 0;
 				}
+				if (changed & MODE_SET) {
+					if (mode)
+						device.gpsdata.fix.mode = mode;
+					else
+						device.gpsdata.fix.mode = 0;
+				}
 				device.gpsdata.set = changed;
 
 				gpsd_report(3, "Signaling status changes.\n");
