@@ -73,6 +73,9 @@ void gpsd_init(struct gps_device_t *session, struct gps_context_t *context, char
     /* necessary in case we start reading in the middle of a GPGSV sequence */
     gpsd_zero_satellites(&session->gpsdata);
 
+    /* clear set history */
+    session->set = 0;
+
     /* initialize things for the packet parser */
     packet_reset(session);
 }
@@ -583,6 +586,8 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	gps_merge_fix(&session->gpsdata.fix, received, &session->gpsdata.newdata);
 	session->gpsdata.set = ONLINE_SET | dopmask | received;
 
+	session->set |= session->gpsdata.set; /* remember the history */
+
 	/* count good fixes */
 	if (session->gpsdata.status > STATUS_NO_FIX) 
 	    session->context->fixcnt++;
